import stream_gears

if __name__ == '__main__':
    stream_gears.upload(
        ["examples/test.mp4"],
        "cookies.json",
        "title",
        171,
        "tag",
        1,
        "source",
        "desc",
        "dynamic",
        "",
        0,
        0,
        0,
        0,
        3,
        [],
        None,
        stream_gears.UploadLine.Bda2,
    )

    stream_gears.upload_by_app(
        ["examples/test.mp4",
         "examples/test2.mp4"],
        "cookies.json",
        "dadad",
        171,
        "演示",
        1,
        "",
        "",
        "",
        "",
        0,
        0,
        0,
        0,
        True,
        False,
        False,
        3,
        [],
        None,
        stream_gears.UploadLine.Qn,
    )
